
/* jshint node: true, loopfunc: true, undef: true, unused: true */

var Route = require('zfinder-route');
var fsExtraPromise = require('fs-extra-promise');
var objectUtils = require('zero-lang/object');
var path = require('path');
var shelljs = require('shelljs');
var zfinderUtils = require('zfinder-utils');

module.exports = function(initOptions) {
    return new Route({
        // properties
        method: 'post', // all|get|post|put|delete|options
        url: '*',
        // methods
        serve: function(options, serverConfig) {
            options = objectUtils.extend({}, initOptions.defaultConfig, options);
            initOptions.fillProperties(this, options);
            var root = serverConfig.root;

            return function(req, res, next) {
                var params = req._params;
                if (params._mw === options.name) {
                    var pathname = decodeURIComponent(req._urlInfo.pathname);
                    if (params.type === 'file') {
                        /*
                         * params:
                         *     type: file
                         *     content: $content
                         */
                        var content = params.content || '';
                        var pathWithOutFilename = pathname.replace(/\/[^\/]+$/, '');
                        var filename = path.join(root, pathname);
                        shelljs.mkdir('-p', path.join(root, pathWithOutFilename));
                        fsExtraPromise.writeFileAsync(filename, content).then(function() {
                            res._JSONRes(
                                zfinderUtils.wrapPathInfo(root, root, pathname)
                            );
                        }, function(err) {
                            next(err);
                        });
                    } else if (params.type === 'directory') {
                        shelljs.mkdir('-p', path.join(root, pathname));
                        res._JSONRes(
                            zfinderUtils.wrapPathInfo(root, root, pathname)
                        );
                    } else {
                        next();
                    }
                } else {
                    next();
                }

            };
        }
    });
};

