
/* jshint node: true, loopfunc: true, undef: true, unused: true */

var path = require('path');
var fsExtraPromise = require('fs-extra-promise');
var Route = require('zfinder-route');
var arrayUtils = require('zero-lang/array');
var objectUtils = require('zero-lang/object');
var zfinderUtils = require('zfinder-utils');

module.exports = function(initOptions) {
    return new Route({
        // properties
        method: 'get', // all|get|post|put|delete|options
        url: '*',
        // methods
        serve: function(options, serverConfig) {
            options = objectUtils.extend({}, initOptions.defaultConfig, options);
            initOptions.fillProperties(this, options);
            var root = serverConfig.root;

            return function(req, res, next) {
                var params = req._params;
                if (params._mw === options.name) {
                    var pathname = decodeURIComponent(req._urlInfo.pathname);
                    if (params.type === 'file') {
                        res._JSONRes(
                            zfinderUtils.wrapFileInfo(root, root, pathname)
                        );
                    } else if (params.type === 'directory') {
                        pathname = path.join(serverConfig.root, pathname);
                        fsExtraPromise.readdirAsync(pathname).then(function(files) {
                            res._JSONRes(arrayUtils.map(files, function(filename) {
                                return zfinderUtils.wrapPathInfo(root, pathname, filename);
                            }));
                        }, function(err) {
                            next(err);
                        });
                    } else {
                        next();
                    }
                } else {
                    next();
                }
            };
        }
    });
};

